# Startup script for Automic Workload Automation Java User Interface (UC4 GUI)
# (c) 2016-2017 Michael A. Lowry <michael.lowry@gmail.com> <michael.lowry@gmail.com>
#
# The most recent version of this script can be found here:
# https://community.automic.com/discussion/comment/3179
#
# Version        Date          Description
# 2              2016.10.26    Write all output, including UCDJ stdout & stderr, to log file.
# 1              2016.08.09    Initial port of BAT script to PowerShell
# 3              2017.09.13    Updated for PowerShell 5.1. Added prompts for missing inputs.

param (
#    [Parameter(Mandatory=$true)][string]$uc4env,
#    [Parameter(Mandatory=$true)][string]$uc4ver,
    [string]$uc4env,
    [string]$uc4ver,
    [string]$jrever
)

$defaultUc4Env = "DEV2"
$defaultUc4Ver = "12.0.3"
$defaultJreVer = "jre1.8.0_141"
$userPrompted  = "No"
$msgprt1       = ""

[void] [System.Reflection.Assembly]::LoadWithPartialName("System.Drawing")
[void] [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
Add-Type AssemblyName System.Windows.Forms

$host.UI.RawUI.BufferSize = new-object System.Management.Automation.Host.Size(512,50)

# Check user-specified environment name.
Switch ($uc4env)
	{
	"EXP"   { $uc4env_desc = "Experimental"}
	"EXP2"  { $uc4env_desc = "Experimental 2"}
	"DEV"   { $uc4env_desc = "Development"}
	"ITE"   { $uc4env_desc = "Integrated Test Env."}
	"PROD"  { $uc4env_desc = "Production"}
    ""      { $msprt1  = " UC4 environment not specified." }
	default { $msprt1  = "Unknown UC4 environment: $uc4env." }
}
if ($msgprt1 -ne "") {
    $message = "$msprt1 Use default ($defaultUc4Env)?"
    $result = [System.Windows.Forms.MessageBox]::Show($message,'Warning','YesNo')
    Write-Host "WARNING: $message"
    if ($result -eq 'No') {
        Write-Host "Aborting."
        cd $cwdlong
        Exit 1 }
    else {
        $userPrompted = "Yes"
        $uc4env = $defaultUc4Env
    }
}
$msgprt1       = ""


# Check user-specified UC4 version. Modify this section to add support for other versions.
Switch ($uc4ver)
	{
	"11"        { $uc4ver = "11.2.5"}
	"12"        { $uc4ver = $defaultUc4Ver }
	default {
        $message = "UC4 version not specified. Use default ($defaultUc4Ver)?"
        $result = [System.Windows.Forms.MessageBox]::Show($message,'Warning','YesNo')
        Write-Host "WARNING: $message"
        if ($result -eq 'No') {
            Write-Host "Aborting."
            cd $cwdlong
            Exit 1 }
        else {
            $userPrompted = "Yes"
            $uc4ver = $defaultUc4Ver
        }
    }
}

# Check user-specified JRE version. Add additional blocks to support Java 6 or 7.
Switch ($jrever)
	{
	"8"         { $jrever = $defaultJreVer }
	"1.8"       { $jrever = $defaultJreVer }
    ""          { $msgprt1  = "JRE version not specified." }
	default     { $msgprt1 = "Unknown JRE version: $jrever." }
    }
if ($msgprt1 -ne "") {
    $message = "$msgprt1 Use default ($defaultJreVer)?"
    $result = [System.Windows.Forms.MessageBox]::Show($message,'Warning','YesNo')
    Write-Host "WARNING: $message"
    if ($result -eq 'No') {
        Write-Host "Aborting."
        cd $cwdlong
        Exit 1 }
    else {
        $userPrompted = "Yes"
        $jrever = $defaultJreVer
    }
}

$datetime = Get-Date -format yyyy.MM.dd_HH.mm.ss
$datetime2 = Get-Date -format "yyyy.MM.dd HH:mm:ss"
$myDocuments = [Environment]::GetFolderPath("MyDocuments")
$configdir = "$myDocuments\UC4\UCDJ\$uc4env\v$uc4ver"
$uc4config = "$configdir\uc4config-$uc4env.xml"
$logindat  = "$configdir\login_dat-$uc4env.xml"
$logdir    = "$configdir\logs"
$logfile   = "$logdir\UCDJ_${uc4env}_log_$datetime.txt"
# Start writing log
$ErrorActionPreference="SilentlyContinue"
Stop-Transcript | out-null
$ErrorActionPreference = "Continue"
Start-Transcript -path $logfile -append
$cwdlong = Get-Location
$fso = New-Object -ComObject Scripting.FileSystemObject
$cwd = $fso.GetFolder($cwdlong)
$scriptdir = $cwd.ShortPath
$userprofile = $env:userprofile
$ucdjdir = "$ScriptDir\UCDJ\v$uc4ver\bin"
$jredir = "$ScriptDir\Java\$jrever"
$username = $env:username

Write-Host "Date & time          : $datetime2"
Write-Host "UC4 Environment      : $uc4env ($uc4env_desc)"
Write-Host "UC4 Version          : $uc4ver"
Write-Host "UCDJ directory       : $ucdjdir"
Write-Host "JRE Version          : $jrever"
Write-Host "JRE directory        : $jredir"
Write-Host "Working dir. (long)  : $cwdlong"
Write-Host "Working dir. (short) : $scriptdir"
Write-Host "User name            : $username"
Write-Host "User home (long)     : $userprofile"
Write-Host "Config. directory    : $configdir"
Write-Host "UI config. XML file  : $uc4config"
Write-Host "UI login data file   : $logindat"
Write-Host "Log directory        : $logdir"
Write-Host "Log file             : $logfile"

# Set paths of required files
$ucdjjar     = "$ucdjdir\ucdj-$uc4env.jar"
$configenv   = "$cwdlong\config\uc4config-$uc4env.xml"
$logindatenv = "$cwdlong\config\login_dat-$uc4env.xml"
$docustubdir = "$cwdlong\UCDJ\v$uc4ver\docu"

Write-Host "Checking for required directories and files."
# Check for existence of required files
if((Test-Path "$jredir\bin\java.exe") -eq 0) {
    $message = "Jave executable file not found: $jredir\bin\java.exe"
    [System.Windows.Forms.MessageBox]::Show($message,'Error','OK')
    Write-Host "ERROR: $message"
    cd $cwdlong
    Exit 1
}

if((Test-Path "$ucdjjar") -eq 0) {
    $message = "Environment-specific UCDJ file $cwdlong\UCDJ\v$uc4ver\bin\ucdj-$uc4env.jar file not found.`nUse default ucdj.jar?"
    $result = [System.Windows.Forms.MessageBox]::Show($message,'Warning','YesNo')
    Write-Host "Warning: ucdj-$uc4env.jar file not found in $cwdlong\UCDJ\v$uc4ver\bin subdirectory."
    if ($result -eq 'No') {
        Write-Host "Aborting."
        cd $cwdlong
        Exit 1 }
    else {
        $userPrompted = "Yes"
        $ucdjjar = "$ucdjdir\ucdj.jar"
        if((Test-Path "$ucdjjar") -eq 0) {
            $message="Default UCDJ file $ucdjdir\ucdj.jar not found."
            [System.Windows.Forms.MessageBox]::Show($message,'Error','OK')
            Write-Host "ERROR: $message"
            cd $cwdlong
            Exit 1
        }
    }
}

if((Test-Path "$configenv") -eq 0) {
    $message = "Environment-specific UI server config file $configenv not found.`nUse default uc4config.xml?"
    $result = [System.Windows.Forms.MessageBox]::Show($message,'Warning','YesNo')
    Write-Host "Warning: Environment-specific UI server config file $configenv not found."
    if ($result -eq 'No') {
        Write-Host "Aborting."
        cd $cwdlong
        Exit 1 }
    else {
        $userPrompted = "Yes"
        $configenv = "$ucdjdir\uc4config.xml"
        $uc4config = "$configdir\uc4config.xml"
        if((Test-Path "$configenv") -eq 0) {
            $message = "Default UI server config file $configenv not found."
            [System.Windows.Forms.MessageBox]::Show($message,'Error','OK')
            Write-Host "ERROR: $message"
            cd $cwdlong
            Exit 1
        }
    }
}

if((Test-Path "$logindatenv") -eq 0) {
    $message = "Environment-specific UI config file $logindatenv not found in $cwdlong\$config directory.`nUse default login_dat.xml?"
    $result = [System.Windows.Forms.MessageBox]::Show($message,'Warning','YesNo')
    Write-Host "Warning: Environment-specific UI login data file $logindatenv not found."
    if ($result -eq 'No') {
        Write-Host "Aborting."
        cd $cwdlong
        Exit 1 }
    else {
        $userPrompted = "Yes"
        $logindatenv = "$ucdjdir\login_dat.xml"
        $logindat  = "$configdir\login_dat.xml"
        if((Test-Path "$logindatenv") -eq 0) {
            $message="Default UI login data file $logindatenv not found."
            [System.Windows.Forms.MessageBox]::Show($message,'Error','OK')
            Write-Host "ERROR: $message"
            cd $cwdlong
            Exit 1
        }
    }
}

if((Test-Path "$docustubdir") -eq 0) {
    $message = "Version-specific documentation stub directory not found: $docustubdir. F1 help may not work.`nContinue?"
    $result = [System.Windows.Forms.MessageBox]::Show($message,'Warning','YesNo')
    Write-Host "ERROR: $message"
    if ($result -eq 'No') {
        Write-Host "Aborting."
        cd $cwdlong
        Exit 1
    }
}

Write-Host "All required directories and files exist."

# Create directories and copy files if necessary
if((Test-Path "$configdir") -eq 0) {
    Write-Host "Directory $configdir does not exist. Creating it."
    New-Item -ItemType Directory -Force -Path $configdir
}
if((Test-Path "$logdir") -eq 0) {
    Write-Host "Directory $logdir does not exist. Creating it."
    New-Item -ItemType Directory -Force -Path $logdir
}
if((Test-Path "$uc4config") -eq 0) {
    Write-Host "Copying initial UC4 UI server config file $configenv to $configdir."
    Copy-Item "$configenv" -Destination "$configdir"
}
if((Test-Path "$logindat") -eq 0) {
    Write-Host "Copying initial UC4 UI login data file $logindatenv to $configdir."
    Copy-Item "$logindatenv" -Destination "$configdir"
}
if((Test-Path "$configdir\docu") -eq 0) {
    Write-Host "Copying default UC4 documentation stub directory $docustubdir to $configdir."
    Copy-Item "$docustubdir" -Destination "$configdir" -Recurse
}

cd $configdir
$userdir  = $configdir
$classpath = "$ucdjdir;$ucdjjar"
$krb5debug = "`"-Dsun.security.krb5.debug=true`""
#$javatmpdir = "`"-Djava.io.tmpdir=$configdir`""
$javaopts  = "`"-Xmx1024m`" `"-XX:+UseG1GC`" `"-XX:MinHeapFreeRatio=40`" `"-XX:MaxHeapFreeRatio=70`" `"-XX:+HeapDumpOnOutOfMemoryError`" `"-XX:HeapDumpPath=$logdir`" `"-Dsun.locale.formatasdefault=true`" `"$krb5debug`" `"-Duser.dir=$userdir`""
$mainclass = "com/uc4/ucdf/UCDialogFactory"
$ucdjopts  = "`"-U$username`" `"-I$uc4config`" `"-O$logindat`""

Write-Host "Java class path      : $classpath"
Write-Host "Java options         : $javaopts"
Write-Host "Java main class      : $mainclass"
Write-Host "UCDJ options         : $ucdjopts"

# If the user was prompted to use a default value, confirm the selected options before starting.
if ($userPrompted -eq "Yes") {
    $message="Start UC4 Java GUI with these options?`n`nUC4 version: $uc4ver`nEnvironment: $uc4env`nJRE version: $jrever"
    $result = [System.Windows.Forms.MessageBox]::Show($message,'Confirmation','YesNo')
    Write-Host $message
    if ($result -eq 'No') {
        Write-Host "Aborting."
        cd $cwdlong
        Exit 1
    }
}
# Start the program
Write-Host "Running command: $jredir\bin\java.exe $javaopts -cp $classpath $mainclass $ucdjopts >>$logfile 2>&1"
Stop-Transcript
# The -RedirectStandardError and -RedirectStandardOutputnot options are not used here because it's not possible to redirect both to the same file.
Start-Process $jredir\bin\java.exe -WindowStyle Hidden -ArgumentList "$javaopts","-cp $classpath","$mainclass","$ucdjopts",">>$logfile","2>&1"
cd $cwdlong